@@ -3,13 +3,13 @@ BEGIN {
   $App::Pocoirc::Status::AUTHORITY = 'cpan:HINRIK';
 }
 BEGIN {
-  $App::Pocoirc::Status::VERSION = '0.36';
+  $App::Pocoirc::Status::VERSION = '0.37';
 }
 
 use strict;
 use warnings FATAL => 'all';
 use Carp;
-use POE::Component::IRC::Common qw(irc_to_utf8 strip_color strip_formatting);
+use IRC::Utils qw(decode_irc strip_color strip_formatting numeric_to_name);
 use POE::Component::IRC::Plugin qw(PCI_EAT_NONE);
 use Scalar::Util qw(looks_like_number);
 
@@ -58,9 +58,21 @@ sub PCI_unregister {
     return 1;
 }
 
+sub verbose {
+    my ($self, $value) = @_;
+    $self->{Verbose} = $value;
+    return;
+}
+
+sub trace {
+    my ($self, $value) = @_;
+    $self->{Trace} = $value;
+    return;
+}
+
 sub _normalize {
     my ($line) = @_;
-    $line = irc_to_utf8($line);
+    $line = decode_irc($line);
     $line = strip_color($line);
     $line = strip_formatting($line);
     return $line;
@@ -148,7 +160,8 @@ sub S_001 {
     my ($self, $irc) = splice @_, 0, 2;
     my $server = ${ $_[0] };
     my $nick = $irc->nick_name();
-    $self->_event_debug($irc, 'S_001', \@_) if $self->{Trace};
+    my $event = 'S_001 ('.numeric_to_name('001').')';
+    $self->_event_debug($irc, 'event', \@_) if $self->{Trace};
     $self->{status}{$irc}->('normal', "Logged in to server $server with nick $nick");
     return PCI_EAT_NONE;
 }
@@ -238,13 +251,6 @@ sub S_quit {
     return PCI_EAT_NONE;
 }
 
-sub S_shutdown {
-    my ($self, $irc) = splice @_, 0, 2;
-    $self->_event_debug($irc, 'S_shutdown', \@_) if $self->{Trace};
-    $self->{status}{$irc}->('normal', 'IRC component shut down');
-    return PCI_EAT_NONE;
-}
-
 sub S_socketerr {
     my ($self, $irc) = splice @_, 0, 2;
     my $reason = _normalize(${ $_[0] });
@@ -286,10 +292,17 @@ sub S_raw_out {
     $self->{status}{$irc}->('debug', ">>> $raw");
     return PCI_EAT_NONE;
 }
+
 sub _default {
     my ($self, $irc, $event) = splice @_, 0, 3;
     return PCI_EAT_NONE if !$self->{Trace};
     return PCI_EAT_NONE if $event =~ /^S_plugin_/;
+
+    if (my ($numeric) = $event =~ /^[SU]_(\d+)$/) {
+        my $name = numeric_to_name($numeric);
+        $event .= " ($name)" if defined $name;
+    }
+
     $self->_event_debug($irc, $event, \@_) if $self->{Trace};
     return PCI_EAT_NONE;
 }
